"use strict";

/**
 * CAK interface
 * @class Cak
 * @namespace PluginInterface
 * @static
 */
PluginInterface.Cak = new function() {
    /*
     ***********************************************************************************************************
     PRIVATE VARIABLES AND FUNCTIONS
     ***********************************************************************************************************
     */

    /**
     * Info callback function
     * @method _callback_cak_function
     * @private
     */
    var _callback_cak_function = null,
    /**
     * Info callback function
     * @method _callback_cak_pin_function
     * @private
     */
    _callback_cak_pin_function = null,
    /**
     * Info callback function
     * @method _callback_cak_content_status_function
     * @private
     */
    _callback_cak_content_status_function = null,
    /**
     * Cak callback
     * @private
     * @method _fCakCallback
     * @param event {Number} Cas event
     * @param jsondata {STRING} Data packed in JSON string
     */
    _fCakCallback = function (event, jsondata) {
        var data = null;
        switch (parseInt(event, 10)) {
            case this.casEvent.CAE_ERROR_POPUP:
            case this.casEvent.CAE_DISPLAY_POPUP:
            case this.casEvent.CAE_REMOVE_POPUP:
            case this.casEvent.CAE_ENTITLEMENT_MESSAGE:
                /** The Popup message can be on multi line => the json parse didn't treat \n but need a escape character */
                data = JSON.parse(jsondata.replace(/\n/g, "\\n"));
                if (_callback_cak_function) {
                    _callback_cak_function(event, data);
                }
                break;
            case this.casEvent.CAE_CAS_ID_MISMATCH:
            case this.casEvent.CAE_BOOT_ERROR:
                if (_callback_cak_function) {
                    _callback_cak_function(event, null);
                }
                break;
            case this.casEvent.CAE_PIN_CODE_RESULT:
                data = JSON.parse(jsondata)
                if (_callback_cak_pin_function) {
                    _callback_cak_pin_function(data.type, data.result);
                }
                break;
            case this.casEvent.CAE_PERSISTENT_DATA:
                data = JSON.parse(jsondata);
                if (_callback_cak_function) {
                    _callback_cak_function(event, data);
                }
                break;
            case this.casEvent.CAE_SMARTCARD_NOT_ACTIVATED:
                if (_callback_cak_function) {
                    _callback_cak_function(event, null);
                }
                break;
            case this.casEvent.CAE_STATUS_CONTENT:
                data = JSON.parse(jsondata)
                if (_callback_cak_content_status_function) {
                    _callback_cak_content_status_function(data);
                }
                if (data.smartcardState == PluginInterface.Cak.smartCardState.CAS_STATUS_SMARTCARD_EXTRACTED) {
                    if (_callback_cak_function) {
                        _callback_cak_function(event, data);
                    }
                }
                break;
            default:
                console.log("cak event " + event + " not handled");
                break;
        }
    };
    /*
     ***********************************************************************************************************
     PRIVILEGED METHODS AND PROPERTIES
     ***********************************************************************************************************
     */
    /**
     * Enumerated error severity types
     * @type {Object}
     * @property errorSeverityTypes
     * @final
     */
    this.errorSeverityTypes = {
        SEVERITY_FATAL: 1,
        SEVERITY_ERROR: 2,
        SEVERITY_WARNING: 3,
        SEVERITY_INFO: 4,
        SEVERITY_INFO_DND: 5
    };
    /**
     * Enumerated error status types
     * @type {Object}
     * @property errorStatusTypes
     * @final
     */
    this.errorStatusTypes = {
        STATUS_ACCESS_CLEAR: 0,
        STATUS_SMARTCARD_ERROR: 1,
        STATUS_SMARTCARD_MUTE: 2,
        STATUS_SMARTCARD_INVALID: 3,
        STATUS_SMARTCARD_BLACKLISTED: 4,
        STATUS_SMARTCARD_SUSPENDED: 5,
        STATUS_SMARTCARD_NEVER_PAIRED: 6,
        STATUS_SMARTCARD_NOT_PAIRED: 7,
        STATUS_SMARTCARD_EXPIRED: 8,
        STATUS_SMARTCARD_NOT_CERTIFIED: 9,
        STATUS_SMARTCARD_INCOMPATIBLE: 10,
        STATUS_ACCESS_GRANTED: 11,
        STATUS_ACCESS_FREE: 12,
        STATUS_ACCESS_DENIED: 13,
        STATUS_ACCESS_BLACKED_OUT: 14,
        STATUS_ACCESS_DENIED_NO_VALID_CREDIT: 15,
        STATUS_ACCESS_DENIED_COPY_PROTECTED: 16,
        STATUS_ACCESS_DENIED_PARENTAL_CONTROL: 17,
        STATUS_ACCESS_DENIED_DIALOG_REQUIRED: 18,
        STATUS_ACCESS_DENIED_PAIRING_REQUIRED: 19,
        STATUS_ACCESS_DENIED_CHIPSET_PAIRING_REQUIRED: 20,
        STATUS_UNKNOW_ERROR: 21,
        STATUS_SECURITY_UPGRADE_REQUIRED: 22,
        STATUS_SMARTCARD_EXTRACTED: 23
    };
    /**
     * Enumerated pin types
     * @type {Object}
     * @property pinType
     * @final
     */
    this.pinType = {
        PCT_PARENTAL_CONTROL: 0,/**< The pin code query targets the parental control CA feature. */
        PCT_PAY_PER_VIEW: 1,    /**< The pin code query targets the purchase CA feature. */
        PCT_MASTER: 2,          /**< The pin code query targets the master pin code, controlling other pin codes. */
        PCT_NEW: 3              /**< The pin code query forwards a new pin code value for the last requested pin code. */

    };
    /**
     * Enumerated pin result types
     * @type {Object}
     * @property pinResultType
     * @final
     */
    this.pinResultType = {
        PCR_FAIL_CARD_OUT: 0,   /**< The smartcard is not inserted or was removed during the operation. */
        PCR_CHECK_OK: 1,        /**< The pin code was verified ok. */
        PCR_FAILED: 2,          /**< The pin code was rejected. */
        PCR_CHANGED: 3,         /**< The pin code was successfully changed. */
        PCR_TYPE_INVALID: 4,    /**< The operation is invalid for the pin code type. */
        PCR_BLOCK: 5            /**< The pin code is currently blocked. */
    };
    /**
     * Enumerated cas event
     * @type {Object}
     * @property casEvent
     * @final
     */
    this.casEvent = {
        CAE_MATURITY_LEVEL: 0, /**< Plug-In to Package. The current maturity level changed. */
        CAE_RUNNING_STATUS: 1, /**< Plug-In to Package. The current running status changed. */
        CAE_STATUS_SCREEN: 2, /**< Plug-In to Package. Status screen navigation changed screen. */
        CAE_PIN_CODE: 3, /**< Plug-In to Package. Pin code validation is requested. */
        CAE_GET_MESSAGE_LIST: 4, /**< Plug-In to Package. Mailbox message list is requested. */
        CAE_DISPLAY_MESSAGE: 5, /**< Plug-In to Package. Mailbox message contents are requested. */
        CAE_INCOMING_MESSAGE: 6, /**< Package to Plug-In. A mailbox message is unread. */
        CAE_ERROR_POPUP: 7, /**< Package to Plug-In. The current error status changed. */
        CAE_STATUS_CONTENT: 8, /**< Package to Plug-In. The current status screen content changed. */
        CAE_MESSAGE_LIST: 9, /**< Package to Plug-In. The current mailbox message list changed. */
        CAE_MESSAGE_CONTENT: 10, /**< Package to Plug-In. Requested mailbox message content arrived. */
        CAE_PIN_CODE_QUERY: 11, /**< Package to Plug-In. Pin code is required to continue. */
        CAE_PIN_CODE_RESULT: 12, /**< Package to Plug-In. Pin code validation result arrived. */
        CAE_GET_UTC_TIME: 13, /**< Package to Plug-In. UTC Time is requested. */
        CAE_CHANGE_PIN_CODE: 14, /**< Package to Plug-In. Pin code change is required to continue. */
        CAE_RESET_IRD: 15, /**< Package to Plug-In. Decoder reset is required "to continue". */
        CAE_FORCE_TUNE: 16, /**< Package to Plug-In. Forced tune is required to continue. */
        CAE_SMARTCARD_NATIONALITY: 17, /**< Package to Plug-In. Smartcard nationality changed. */
        CAE_SOFTWARE_UPDATE: 18, /**< Package to Plug-In. A software search returned results. */
        CAE_DO_DOWNLOAD: 19, /**< Plug-In to Package. Do download the software update. */
        CAE_RECONNECT: 20, /**< Package to Plug-In. Reconnect to current service. */
        CAE_UPDATE_ARRIVAL_DATE: 21, /**< Plug-In to Package. Indicate that Message has been display */
        CAE_MARK_AS_READ: 22, /**< Plug-In to Package. Indicate that Message has been read */
        CAE_ZONE_LOCK: 23, /**< Package to Plug-In. Indicate that Zone Lock status has changed */
        CAE_ENTITLEMENT_MESSAGE: 24, /**< Package to Plug-In. Notify Entitlement Message */
        CAE_CAS_ID_MISMATCH: 25,
        CAE_DISPLAY_POPUP: 26,
        CAE_REMOVE_POPUP: 27,
        CAE_SMARTCARD_NOT_ACTIVATED: 28,
        CAE_READ_PERSISTENT_DATA: 29,
        CAE_PERSISTENT_DATA: 30,
        CAE_WRITE_PERSISTENT_DATA: 31,
        CAE_BOOT_ERROR: 32
    };
    /**
     * Enumerated smart card state
     * @type {Object}
     * @property smartCardState
     * @final
     */
    this.smartCardState = {
        CAS_STATUS_SMARTCARD_OK: 0,
        CAS_STATUS_SMARTCARD_ERROR: 1,
        CAS_STATUS_SMARTCARD_MUTE: 2,
        CAS_STATUS_SMARTCARD_INVALID: 3,
        CAS_STATUS_SMARTCARD_BLACKLISTED: 4,
        CAS_STATUS_SMARTCARD_SUSPENDED: 5,
        CAS_STATUS_SMARTCARD_NEVER_PAIRED: 6,
        CAS_STATUS_SMARTCARD_NOT_PAIRED: 7,
        CAS_STATUS_SMARTCARD_EXPIRED: 8,
        CAS_STATUS_SMARTCARD_NOT_CERTIFIED: 9,
        CAS_STATUS_SMARTCARD_INCOMPATIBLE: 10,
        CAS_STATUS_SMARTCARD_EXTRACTED: 11
    };
    /**
     * Enumerated smart card credit valid statuses
     * @type {Object}
     * @property smartCardCreditValid
     * @final
     */
    this.smartCardCreditValid = {
        SC_CREDIT_INFO_NOT_AVAILABLE: '0',
        SC_CREDIT_NOT_ACTIVATED: '1',
        SC_CREDIT_WILL_BE_ACTIVATED: '2',
        SC_CREDIT_EXPIRED: '3',
        SC_CREDIT_VALID_UNTIL_DATE: '4'
    };
    /**
     * Enumerated IRQ types
     * @type {Object}
     * @property irqTypes
     * @final
     */
    this.irqTypes = {
        IRD_COMMAND_RESET_PIN: 1,
        IRD_COMMAND_RESTORE_FACTORY_SETTINGS: 2
    };
    /**
     * Initialize cak
     * @method fInitCak
     * @static
     */
    this.fInitCak = function () {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.cakNotification = _fCakCallback.bind(this);
    };

    /**
     * Set CAK Callbacks
     * @method fSetCallbacks
     * @param callback {Function} Cak UI callback function
     * @param callback_pin {Function} Cak UI callback function
     * @static
     */
    this.fSetCallbacks = function (callback, callback_pin) {
        _callback_cak_function = callback;
        _callback_cak_pin_function = callback_pin;
    };

    /**
     * Gets cak information object
     * @method fGetCakInfo
     * @param asyncClbck {Function} If not null, cak info is refreshed and returned via this function
     * @returns {Object} CAK info as JSON object
     * @static
     */
    this.fGetCakInfo = function (asyncClbck) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return PluginInterface.Mappings.Dummies.Cak.getCakInfo();
        }
        if (asyncClbck) {
            _callback_cak_content_status_function = asyncClbck;
            PluginInterface.Setup.stb.getCakInfo("1");
        }
        else {
            var str = PluginInterface.Setup.stb.getCakInfo("");
            return JSON.parse(str);
        }
    };

    /**
     * Check or change smart card pin code depending on the type
     * @method fCakPinCode
     * @param type {Number} Enumerated pin types (see pinType)
     * @param value {String} pin code
     * @async
     * @static
     */
    this.fCakPinCode = function (type, value) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            switch (type) {
                case this.pinType.PCT_MASTER:
                    if (value == '0000') {
                        setTimeout(function () {
                            if (_callback_cak_pin_function) _callback_cak_pin_function(type, PluginInterface.Cak.pinResultType.PCR_CHECK_OK)
                        }, 500);
                    }
                    else {
                        setTimeout(function () {
                            if (_callback_cak_pin_function) _callback_cak_pin_function(type, PluginInterface.Cak.pinResultType.PCR_FAILED)
                        }, 500);
                    }
                    break;
                case this.pinType.PCT_NEW:
                    setTimeout(function () {
                        if (_callback_cak_pin_function) _callback_cak_pin_function(type, PluginInterface.Cak.pinResultType.PCR_CHANGED)
                    }, 500);
                    break;
            }
            return;
        }
        PluginInterface.Setup.stb.cakPinCode("" + type, "" + value);
    };

    /**
     * Confirms that entitlement message was viewed by user
     * @method fConfirmEntitlementMessage
     * @param id {Number} message id
     * @param frequency {Number} 0: once, 1: daily, 2: always
     * @param utc_time {Number} message time
     * @static
     */
    this.fConfirmEntitlementMessage = function (id, frequency, utc_time) {
        if (PluginInterface.Mappings.Globals.gPCTest) {
            return;
        }
        PluginInterface.Setup.stb.confirmEntitlementMessage("" + id, "" + frequency, "" + utc_time);
    };
};
